﻿
Imports System
Imports System.IO
Imports System.Text
Imports System.IO.Ports
Imports System.Collections
Imports System.Threading
Imports System.ComponentModel

#Region "License"
'*****************************************************************************************
'                           EIGHT SERVOS CONTROLLER v1.0.1
'*****************************************************************************************
'   Copyright (C) 2013  Marcello Pinna
'   all rigts reserved
'   September 15, 2013
'*****************************************************************************************
'                                     LICENSE
'Redistribution and use in source and binary forms, with or without modification, are
'permitted provided that the following conditions are met: 
'
'Redistributions of source code must retain the above copyright notice, this list of
'conditions and the following disclaimer. 

'Redistributions in binary form must reproduce the above copyright notice, this list 
'of conditions and the following disclaimer in the documentation and/or other materials 
'provided with the distribution. 

'The name of the author may not be used to endorse or promote products derived from this
' software without specific prior written permission.

'THIS SOFTWARE IS PROVIDED BY THE AUTHOR “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, 
'INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
'FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR 
'ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
'(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
'USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
'LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
'OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
'POSSIBILITY OF SUCH DAMAGE.
'*****************************************************************************************
#End Region

Public Class frmMain

#Region "Variables Manager"
    ''' <summary>
    ''' List of variables used in this program
    ''' </summary>
    Dim comm As Array
    Dim Even As String
    Dim Odd As String
    Dim None As String
    Dim Mark As String
    Dim Space As String
    Dim Setting As String
    Dim Index As Integer
    Dim Pulse As Integer
    Dim Servo_pos As Integer
    Dim value As Integer = 150
    Dim ExtThread As System.Threading.Thread
#End Region

#Region "User's COM Setting Manager"
    ''' <summary>
    ''' Load port COM settings saved by user
    ''' </summary>
    Private Sub Load_Settings()
        Dim opendlg As New OpenFileDialog()
        If opendlg.ShowDialog = DialogResult.OK Then
            Dim stream As New StreamReader("Settings.txt")
            Dim sLine As String = ""
            Dim arrText As New ArrayList()
            Do
                sLine = stream.ReadLine()
                If Not sLine Is Nothing Then
                    arrText.Add(sLine)
                End If
            Loop Until sLine Is Nothing
            stream.Close()
            cboPort.Text = arrText(0)
            cboBaudrate.Text = arrText(1)
            cboParity.Text = arrText(2)
            cboFormat.Text = arrText(3)
            cboStopbit.Text = arrText(4)
            txtDelayTX.Text = arrText(5)
            txtDelayRX.Text = arrText(6)
        End If
    End Sub

    ''' <summary>
    ''' Save port COM settings choosen by user
    ''' </summary>
    Private Sub Save_Settings()
        Dim Lastvalue(6) As String
        Lastvalue(0) = cboPort.Text
        Lastvalue(1) = cboBaudrate.Text
        Lastvalue(2) = cboParity.Text
        Lastvalue(3) = cboFormat.Text
        Lastvalue(4) = cboStopbit.Text
        Lastvalue(5) = txtDelayTX.Text
        Lastvalue(6) = txtDelayRX.Text
        Dim savedlg As New SaveFileDialog()
        If savedlg.ShowDialog = DialogResult.OK Then
            Dim stream As New StreamWriter("Settings.txt")
            Dim i As Integer
            For i = 0 To Lastvalue.Length - 1
                stream.WriteLine(Lastvalue(i))
            Next
            stream.Close()
        End If
    End Sub
#End Region

#Region "Update Setting"
    ''' <summary>
    ''' Update Setting label with new values
    ''' </summary>
    Private Sub Update_Setting()
        Setting = cboPort.Text & "," & cboBaudrate.Text & "," & cboParity.Text & "," & cboFormat.Text & "," & cboStopbit.Text
        lblSetting.Text = "RS232 Setting: " & Setting
    End Sub
#End Region

#Region "Servo Position"
    ''' <summary>
    ''' Update Servo Position label with new values
    ''' </summary>
    Private Sub Servo_Position()
        Servo_pos = ((150 - txtPulse.Text * 100) * 9 / 10) + 90
        lblPosition.Text = "    Servo " & Index & " " & "position = " & Servo_pos & "°"
        If Servo_pos > 90 Then
            lblDirection.Text = "   Right"
        ElseIf Servo_pos = 90 Then
            lblDirection.Text = "   Center"
        ElseIf Servo_pos < 90 Then
            lblDirection.Text = "   Left"
        End If
    End Sub
#End Region

#Region "RichTextBox Manager"
    ''' <summary>
    ''' Enable or disable buffers clearing item
    ''' </summary>
    Private Sub rtbDisplay_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rtbDisplay.TextChanged
        If rtbDisplay.Text <> "" Then
            itemClearBuffers.Enabled = True
        Else
            itemClearBuffers.Enabled = False
        End If
    End Sub
#End Region

#Region "Comboboxes Manager"
    ''' <summary>
    ''' All Comboboxes routines
    ''' </summary>
    Private Sub Enable_All_Comboboxes()
        For Each ctrl As Control In Me.boxRS232.Controls
            If TypeOf ctrl Is ComboBox Then
                ctrl.Enabled = True
            End If
        Next
    End Sub

    Private Sub Disable_All_Comboboxes()
        For Each ctrl As Control In Me.boxRS232.Controls
            If TypeOf ctrl Is ComboBox Then
                ctrl.Enabled = False
            End If
        Next
    End Sub

    Private Sub cboPort_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboPort.SelectedIndexChanged
        Update_Setting()
    End Sub

    Private Sub cboBaudrate_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboBaudrate.SelectedIndexChanged
        Update_Setting()
    End Sub

    Private Sub cboParity_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboParity.SelectedIndexChanged
        Update_Setting()
    End Sub

    Private Sub cboFormat_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboFormat.SelectedIndexChanged
        Update_Setting()
    End Sub

    Private Sub cboStopbit_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboStopbit.SelectedIndexChanged
        Update_Setting()
    End Sub
#End Region

#Region "Checkboxes Manager"
    ''' <summary>
    ''' All Checkboxes routines
    ''' </summary>
    Private Sub Enable_All_Checkboxes()
        For Each ctrl As Control In Me.boxController.Controls
            If TypeOf ctrl Is CheckBox Then
                ctrl.Enabled = True
            End If
        Next
    End Sub

    Private Sub Disable_All_Checkboxes()
        For Each ctrl As Control In Me.boxController.Controls
            If TypeOf ctrl Is CheckBox Then
                ctrl.Enabled = False
            End If
        Next
    End Sub

    Private Sub Check_All_Checkboxes()
        For Each Ctrl As Control In Me.boxController.Controls
            If TypeOf Ctrl Is CheckBox Then
                CType(Ctrl, CheckBox).Checked = True
            End If
        Next
    End Sub

    Private Sub UnCheck_All_Checkboxes()
        For Each Ctrl As Control In Me.boxController.Controls
            If TypeOf Ctrl Is CheckBox Then
                CType(Ctrl, CheckBox).Checked = False
            End If
        Next
    End Sub

    Private Sub chkBox1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox1.CheckedChanged
        If chkBox1.Checked Then
            Slider1.Enabled = True
            tltSliderValue.SetToolTip(Slider1, (Slider1.Value / 100).ToString("0.00"))
        Else
            Slider1.Enabled = False
        End If
    End Sub

    Private Sub chkBox2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox2.CheckedChanged
        If chkBox2.Checked Then
            Slider2.Enabled = True
            tltSliderValue.SetToolTip(Slider2, (Slider2.Value / 100).ToString("0.00"))
        Else
            Slider2.Enabled = False
        End If
    End Sub

    Private Sub chkBox3_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox3.CheckedChanged
        If chkBox3.Checked Then
            Slider3.Enabled = True
            tltSliderValue.SetToolTip(Slider3, (Slider3.Value / 100).ToString("0.00"))
        Else
            Slider3.Enabled = False
        End If
    End Sub

    Private Sub chkBox4_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox4.CheckedChanged
        If chkBox4.Checked Then
            Slider4.Enabled = True
            tltSliderValue.SetToolTip(Slider4, (Slider4.Value / 100).ToString("0.00"))
        Else
            Slider4.Enabled = False
        End If
    End Sub

    Private Sub chkBox5_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox5.CheckedChanged
        If chkBox5.Checked Then
            Slider5.Enabled = True
            tltSliderValue.SetToolTip(Slider5, (Slider5.Value / 100).ToString("0.00"))
        Else
            Slider5.Enabled = False
        End If
    End Sub

    Private Sub chkBox6_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox6.CheckedChanged
        If chkBox6.Checked Then
            Slider6.Enabled = True
            tltSliderValue.SetToolTip(Slider6, (Slider6.Value / 100).ToString("0.00"))
        Else
            Slider6.Enabled = False
        End If
    End Sub

    Private Sub chkBox7_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox7.CheckedChanged
        If chkBox7.Checked Then
            Slider7.Enabled = True
            tltSliderValue.SetToolTip(Slider7, (Slider7.Value / 100).ToString("0.00"))
        Else
            Slider7.Enabled = False
        End If
    End Sub

    Private Sub chkBox8_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkBox8.CheckedChanged
        If chkBox8.Checked Then
            Slider8.Enabled = True
            tltSliderValue.SetToolTip(Slider8, (Slider8.Value / 100).ToString("0.00"))
        Else
            Slider8.Enabled = False
        End If
    End Sub
#End Region

#Region "Items Manager"
    ''' <summary>
    ''' All Items routines
    ''' </summary>
    Private Sub itemLoadSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemLoadSettings.Click
        Load_Settings()
    End Sub

    Private Sub itemSaveSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemSaveSettings.Click
        Save_Settings()
    End Sub

    Private Sub itemExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemExit.Click
        If SerialPort1.IsOpen Then
            SerialPort1.DiscardOutBuffer()
            SerialPort1.DiscardInBuffer()
            SerialPort1.Close()
        End If
        Me.Close()
    End Sub

    Private Sub itemConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemConnect.Click
        comm_Open()
        If SerialPort1.IsOpen Then
            txtIndex.Enabled = True
            txtPulse.Enabled = True
            txtDelayTX.Enabled = False
            txtDelayTX.BackColor = Me.BackColor
            txtDelayRX.Enabled = False
            txtDelayRX.BackColor = Me.BackColor
            itemLoadSettings.Enabled = False
            itemSaveSettings.Enabled = False
            itemConnect.Enabled = False
            itemDisconnect.Enabled = True
            Disable_All_Comboboxes()
            Enable_All_Checkboxes()
            btnAllEnabled.Enabled = True
            btnAllDisabled.Enabled = True
            btnAllCentered.Enabled = True
            btnLoad.Enabled = True
            btnSave.Enabled = True
            btnOn.Enabled = True
            Setting = cboPort.Text & "," & cboBaudrate.Text & "," & cboParity.Text & "," & cboFormat.Text & "," & cboStopbit.Text
            lblSetting.Text = "RS232 Setting: " & Setting
            lblStatus.Text = "   Status: CONNECTED"
        Else
            txtIndex.Enabled = False
            txtPulse.Enabled = False
            txtDelayTX.Enabled = True
            txtDelayTX.BackColor = Color.White
            txtDelayRX.Enabled = True
            txtDelayRX.BackColor = Color.White
            itemLoadSettings.Enabled = True
            itemSaveSettings.Enabled = True
            itemConnect.Enabled = True
            itemDisconnect.Enabled = False
            Enable_All_Comboboxes()
            Disable_All_Checkboxes()
            btnAllEnabled.Enabled = False
            btnAllDisabled.Enabled = False
            btnAllCentered.Enabled = False
            btnLoad.Enabled = False
            btnSave.Enabled = False
            btnOn.Enabled = False
            Setting = cboPort.Text & "," & cboBaudrate.Text & "," & cboParity.Text & "," & cboFormat.Text & "," & cboStopbit.Text
            lblSetting.Text = "RS232 Setting: " & Setting
            lblStatus.Text = "   Status: Disconnected"
        End If
    End Sub

    Private Sub itemDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemDisconnect.Click
        If SerialPort1.IsOpen Then
            SerialPort1.DiscardInBuffer()
            SerialPort1.Close()
            txtIndex.Clear()
            txtIndex.Enabled = False
            txtPulse.Clear()
            txtPulse.Enabled = False
            txtDelayTX.Enabled = True
            txtDelayTX.BackColor = Color.White
            txtDelayRX.Enabled = True
            txtDelayRX.BackColor = Color.White
            itemLoadSettings.Enabled = True
            itemSaveSettings.Enabled = True
            itemConnect.Enabled = True
            itemDisconnect.Enabled = False
            Enable_All_Comboboxes()
            UnCheck_All_Checkboxes()
            Disable_All_Checkboxes()
            btnAllEnabled.Enabled = False
            btnAllDisabled.Enabled = False
            btnAllCentered.Enabled = False
            btnLoad.Enabled = False
            btnSave.Enabled = False
            btnOn.Enabled = False
            btnOff.Enabled = False
            lblStatus.Text = "   Status: Disconnected"
        End If
    End Sub

    Private Sub ItemClearBuffers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemClearBuffers.Click
        txtIndex.Clear()
        txtPulse.Clear()
        rtbDisplay.Clear()
    End Sub

    Private Sub itemAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles itemAbout.Click
        frmAbout.Show()
    End Sub
#End Region

#Region "Sliders Manager"
    ''' <summary>
    ''' All Sliders routines
    ''' </summary>
    Private Sub Disable_All_Sliders()
        For Each ctrl As Control In Me.boxController.Controls
            If TypeOf ctrl Is TrackBar Then
                ctrl.Enabled = False
            End If
        Next
    End Sub

    Private Sub Center_All_Sliders()
        Try
            rtbDisplay.Clear()
            With rtbDisplay
                .SelectionColor = Color.DimGray
                .AppendText("...moving all servos to central position  " & vbCr & vbCr)
                .ScrollToCaret()
            End With
            For Each ctrl As Control In Me.boxController.Controls
                If TypeOf ctrl Is TrackBar Then
                    CType(ctrl, TrackBar).Value = value
                    RS232_SendData_Int(ctrl.Tag)
                    Thread.Sleep(Val(txtDelayTX.Text))
                    RS232_SendData_Int(value)
                    Thread.Sleep(Val(txtDelayTX.Text))
                    txtIndex.Clear()
                    txtPulse.Clear()
                    tltSliderValue.SetToolTip(CType(ctrl, TrackBar), (CType(ctrl, TrackBar).Value / 100).ToString("0.00"))
                End If
            Next
        Catch ex As Exception
            MsgBox("Data reading error", MsgBoxStyle.Critical + MsgBoxStyle.OkOnly)
        End Try
        Exit Sub
    End Sub

    Private Sub Save_Servos_Configuration()
        Dim Lastvalue(7) As Integer
        Lastvalue(0) = Slider1.Value
        Lastvalue(1) = Slider2.Value
        Lastvalue(2) = Slider3.Value
        Lastvalue(3) = Slider4.Value
        Lastvalue(4) = Slider5.Value
        Lastvalue(5) = Slider6.Value
        Lastvalue(6) = Slider7.Value
        Lastvalue(7) = Slider8.Value
        Dim savedlg As New SaveFileDialog()
        If savedlg.ShowDialog = DialogResult.OK Then
            Dim stream As New StreamWriter("Values.txt")
            Dim i As Integer
            For i = 0 To Lastvalue.Length - 1
                stream.WriteLine(Lastvalue(i))
            Next
            stream.Close()
        End If
    End Sub

    Private Sub Load_Servos_Configuration()
        Dim opendlg As New OpenFileDialog()
        If opendlg.ShowDialog = DialogResult.OK Then
            Dim stream As New StreamReader("Values.txt")
            Dim sLine As String = ""
            Dim arrText As New ArrayList()
            Do
                sLine = stream.ReadLine()
                If Not sLine Is Nothing Then
                    arrText.Add(sLine)
                End If
            Loop Until sLine Is Nothing
            stream.Close()
            Slider1.Value = Val(arrText(0))
            Slider2.Value = Val(arrText(1))
            Slider3.Value = Val(arrText(2))
            Slider4.Value = Val(arrText(3))
            Slider5.Value = Val(arrText(4))
            Slider6.Value = Val(arrText(5))
            Slider7.Value = Val(arrText(6))
            Slider8.Value = Val(arrText(7))
            Try
                rtbDisplay.Clear()
                With rtbDisplay
                    .SelectionColor = Color.DimGray
                    .AppendText("...moving all servos to last configuration saved" & vbCr & vbCr)
                    .ScrollToCaret()
                End With
                For Each ctrl As Control In Me.boxController.Controls
                    If TypeOf ctrl Is TrackBar Then
                        CType(ctrl, TrackBar).Value = CType(ctrl, TrackBar).Value
                        RS232_SendData_Int(ctrl.Tag)
                        Thread.Sleep(Val(txtDelayTX.Text))
                        RS232_SendData_Int(CType(ctrl, TrackBar).Value)
                        Thread.Sleep(Val(txtDelayTX.Text))
                        txtIndex.Clear()
                        txtPulse.Clear()
                        tltSliderValue.SetToolTip(CType(ctrl, TrackBar), (CType(ctrl, TrackBar).Value / 100).ToString("0.00"))
                    End If
                Next
            Catch ex As Exception
                MsgBox("Data reading error", MsgBoxStyle.Critical + MsgBoxStyle.OkOnly)
            End Try
            Exit Sub
        End If
    End Sub

    Private Sub Slider1_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider1.Scroll
        Index = Slider1.Tag
        Pulse = Slider1.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider1, (Slider1.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider2_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider2.Scroll
        Index = Slider2.Tag
        Pulse = Slider2.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider2, (Slider2.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider3_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider3.Scroll
        Index = Slider3.Tag
        Pulse = Slider3.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider3, (Slider3.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider4_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider4.Scroll
        Index = Slider4.Tag
        Pulse = Slider4.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider4, (Slider4.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider5_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider5.Scroll
        Index = Slider5.Tag
        Pulse = Slider5.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider5, (Slider5.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider6_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider6.Scroll
        Index = Slider6.Tag
        Pulse = Slider6.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider6, (Slider6.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider7_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider7.Scroll
        Index = Slider7.Tag
        Pulse = Slider7.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider7, (Slider7.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub

    Private Sub Slider8_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider8.Scroll
        Index = Slider8.Tag
        Pulse = Slider8.Value
        txtIndex.Text = Index
        txtPulse.Text = (Pulse / 100).ToString("0.00")
        RS232_SendData_Int(Index)
        Thread.Sleep(Val(txtDelayTX.Text))
        RS232_SendData_Int(Pulse)
        Thread.Sleep(Val(txtDelayTX.Text))
        With rtbDisplay
            .SelectionColor = Color.Black
            .AppendText("TX : " & vbTab & Index & "   " & (Pulse / 100).ToString("0.00") & vbCr)
            .ScrollToCaret()
        End With
        tltSliderValue.SetToolTip(Slider8, (Slider8.Value / 100).ToString("0.00"))
        Servo_Position()
    End Sub
#End Region

#Region "Buttons Manager"
    ''' <summary>
    ''' All Buttons routines
    ''' </summary>
    Private Sub btnAllEnabled_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAllEnabled.Click
        'This event causes the enabling of all the servos. See Checkboxes Manager
        Check_All_Checkboxes()
    End Sub

    Private Sub btnAllDisabled_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAllDisabled.Click
        'This event causes the disabling of all the servos. See Checkboxes Manager
        UnCheck_All_Checkboxes()
    End Sub

    Private Sub btnAllCentered_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAllCentered.Click
        Center_All_Sliders()
    End Sub

    Private Sub btnLoad_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoad.Click
        Load_Servos_Configuration()
    End Sub

    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
        Save_Servos_Configuration()
    End Sub

    Private Sub btnOn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOn.Click
        'External thread for loop routine. Loop will never stop without it
        ExtThread = New Threading.Thread(AddressOf Me.Servos_Loop_On)
        ExtThread.Start()
    End Sub

    Private Sub btnOff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOff.Click
        SerialPort1.DiscardOutBuffer()
        ExtThread.Abort()
        For Each ctrl As Control In Me.boxController.Controls
            If TypeOf ctrl Is TrackBar Then
                CType(ctrl, TrackBar).Value = CType(ctrl, TrackBar).Value
                RS232_SendData_Int(ctrl.Tag)
                Thread.Sleep(Val(txtDelayTX.Text))
                RS232_SendData_Int(CType(ctrl, TrackBar).Value)
                Thread.Sleep(Val(txtDelayTX.Text))
                txtIndex.Clear()
                txtPulse.Clear()
                tltSliderValue.SetToolTip(CType(ctrl, TrackBar), (CType(ctrl, TrackBar).Value / 100).ToString("0.00"))
            End If
        Next
        Delay()
        itemDisconnect.Enabled = True
        Enable_All_Checkboxes()
        btnAllEnabled.Enabled = True
        btnAllDisabled.Enabled = True
        btnAllCentered.Enabled = True
        btnLoad.Enabled = True
        btnSave.Enabled = True
        btnOn.Enabled = True
        btnOff.Enabled = False
        rtbDisplay.Clear()
        With rtbDisplay
            .SelectionColor = Color.DimGray
            .AppendText("...loop stopped" & vbCr & vbCr)
            .ScrollToCaret()
        End With
    End Sub
#End Region

#Region "Set Port Default"
    ''' <summary>
    ''' Set port COM with default values (COMx,19200,None,8,1)
    ''' </summary>
    Private Sub Set_Port_Default()
        'Get all com ports available
        comm = IO.Ports.SerialPort.GetPortNames()
        For i = 0 To UBound(comm)
            cboPort.Items.Add(comm(i))
        Next
        'Fill cboBaudrate combobox with all values listed below
        cboBaudrate.Items.Add(110)
        cboBaudrate.Items.Add(300)
        cboBaudrate.Items.Add(600)
        cboBaudrate.Items.Add(1200)
        cboBaudrate.Items.Add(2400)
        cboBaudrate.Items.Add(4800)
        cboBaudrate.Items.Add(9600)
        cboBaudrate.Items.Add(14400)
        cboBaudrate.Items.Add(19200)
        cboBaudrate.Items.Add(28800)
        cboBaudrate.Items.Add(38400)
        cboBaudrate.Items.Add(56000)
        cboBaudrate.Items.Add(57600)
        cboBaudrate.Items.Add(115200)
        'Fill cboParity combobox with all values listed below
        cboParity.Items.Add("Even")
        cboParity.Items.Add("Odd")
        cboParity.Items.Add("None")
        cboParity.Items.Add("Mark")
        cboParity.Items.Add("Space")
        'Fill cboFormat combobox with all values listed below
        cboFormat.Items.Add(5)
        cboFormat.Items.Add(6)
        cboFormat.Items.Add(7)
        cboFormat.Items.Add(8)
        'Fill cboStopbit combobox with all values listed below
        cboStopbit.Items.Add(1)
        cboStopbit.Items.Add(2)
        'Set cboPort to the port COM choosen by user 
        cboPort.Text = cboPort.Items.Item(0)
        'Set cboBaudrate to 19200
        cboBaudrate.Text = cboBaudrate.Items.Item(8)
        'Set cboParity to None
        cboParity.Text = cboParity.Items.Item(2)
        'Set cboFormat to 8
        cboFormat.Text = cboFormat.Items.Item(3)
        'Set cboStopbit to 1
        cboStopbit.Text = cboStopbit.Items.Item(0)
        'Set TX and RX Delay Textboxes
        txtDelayTX.Text = 100
        txtDelayRX.Text = 10
    End Sub
#End Region

#Region "Form Main Manager"
    ''' <summary>
    ''' Main form routines
    ''' </summary>
    Private Sub frmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Try
            Set_Port_Default()
        Catch ex As Exception
            MsgBox("No Serial Ports have been found", MsgBoxStyle.Information + MsgBoxStyle.OkOnly, "Info")
        End Try
        Update_Setting()
        lblStatus.Text = "   Status: Disconnected"
        txtIndex.Enabled = False
        txtPulse.Enabled = False
        itemLoadSettings.Enabled = True
        itemSaveSettings.Enabled = True
        itemDisconnect.Enabled = False
        itemClearBuffers.Enabled = False
        Disable_All_Checkboxes()
        Disable_All_Sliders()
        btnAllEnabled.Enabled = False
        btnAllDisabled.Enabled = False
        btnAllCentered.Enabled = False
        btnLoad.Enabled = False
        btnSave.Enabled = False
        btnOn.Enabled = False
        btnOff.Enabled = False
        'DON'T REMOVE THE FOLLOWING LINE: in this way, CheckForIllegalCrossThreadCalls is not performed
        Control.CheckForIllegalCrossThreadCalls = False
    End Sub

    Private Sub frmMain_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If SerialPort1.IsOpen Then
            SerialPort1.DiscardOutBuffer()
            SerialPort1.DiscardInBuffer()
            SerialPort1.Close()
        End If
    End Sub
#End Region

#Region "Open port"
    ''' <summary>
    ''' Open port communication
    ''' </summary>
    Private Function comm_Open() As Boolean
        Try
            SerialPort1.PortName = cboPort.Text
            SerialPort1.BaudRate = cboBaudrate.Text
            SerialPort1.Parity = cboParity.SelectedValue
            SerialPort1.DataBits = cboFormat.Text
            SerialPort1.StopBits = cboStopbit.Text
            SerialPort1.ReadTimeout = ((Val(txtDelayRX.Text)) * 2)
            If SerialPort1.IsOpen = True Then
                SerialPort1.Close()
            End If
            SerialPort1.Open()
            Return True
        Catch ex As Exception
            MsgBox(cboPort.Text & " " & "Port access denied", MsgBoxStyle.Critical + MsgBoxStyle.OkOnly)
            Return False
        End Try
    End Function
#End Region

#Region "Write data"
    ''' <summary>
    ''' Send byte to RS232
    ''' </summary>
    Public Sub RS232_SendData_Int(ByVal message As String)
        Dim byte_to_send(1) As Byte
        byte_to_send(1) = Val(message)
        Try
            SerialPort1.Write(byte_to_send, 1, 1)
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Information + MsgBoxStyle.OkOnly, "Info")
        End Try

    End Sub
#End Region

#Region "DataReceived"
    ''' <summary>
    ''' Event handler for the DataReceived
    ''' </summary>
    Private Sub DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        rtbDisplay.Invoke(New myDelegate(AddressOf updateRTB), New Object() {})
    End Sub
#End Region

#Region "UpdateRTB"
    ''' <summary>
    ''' Delegate and subroutine to update the RichTextBox control
    ''' </summary>
    Public Delegate Sub myDelegate()
    Public Sub updateRTB()
        Try
            With rtbDisplay
                .SelectionColor = Color.Blue
                .AppendText("RX :  " & vbTab)
                .AppendText(SerialPort1.ReadByte & "   ")
                Thread.Sleep(Val(txtDelayRX.Text))
                .AppendText((SerialPort1.ReadByte / 100).ToString("0.00") & "  ")
                Thread.Sleep(Val(txtDelayRX.Text))
                .AppendText(vbCr & vbCr)
                .ScrollToCaret()
            End With
        Catch ex As Exception
            MsgBox("Data reading error", MsgBoxStyle.Critical + MsgBoxStyle.OkOnly)
            SerialPort1.DiscardInBuffer()
        End Try
        Exit Sub
    End Sub
#End Region

#Region "Delay"
    ''' <summary>
    ''' Timer Delay routine
    ''' </summary>
    Private Sub Delay()
        Timer1.Enabled = True
        Timer1.Interval = (Val(txtDelayTX.Text))
        Timer1.Start()
        Application.DoEvents()
        Timer1.Enabled = False
    End Sub
#End Region

#Region "Servos Loop"
    ''' <summary>
    ''' Servos Loop routine
    ''' </summary>
    Private Sub Servos_Loop_On()
        itemDisconnect.Enabled = False
        Disable_All_Sliders()
        UnCheck_All_Checkboxes()
        Disable_All_Checkboxes()
        btnAllEnabled.Enabled = False
        btnAllDisabled.Enabled = False
        btnAllCentered.Enabled = False
        btnLoad.Enabled = False
        btnSave.Enabled = False
        btnOn.Enabled = False
        btnOff.Enabled = True
        rtbDisplay.Clear()
        With rtbDisplay
            .SelectionColor = Color.DimGray
            .AppendText("...starting loop" & vbCr & vbCr)
            .ScrollToCaret()
        End With
        Do
            For Each ctrl As Control In Me.boxController.Controls
                If TypeOf ctrl Is TrackBar Then
                    CType(ctrl, TrackBar).Value = CType(ctrl, TrackBar).Value
                    RS232_SendData_Int(ctrl.Tag)
                    Thread.Sleep(Val(txtDelayTX.Text))
                    RS232_SendData_Int(CType(ctrl, TrackBar).Value)
                    Thread.Sleep(Val(txtDelayTX.Text))
                    txtIndex.Clear()
                    txtPulse.Clear()
                    tltSliderValue.SetToolTip(CType(ctrl, TrackBar), (CType(ctrl, TrackBar).Value / 100).ToString("0.00"))
                End If
            Next
            Delay()
            SerialPort1.DiscardOutBuffer()
        Loop
        Exit Sub
    End Sub
#End Region

    



End Class
